@@ -69,7 +69,73 @@ class TestHighLine < Test::Unit::TestCase
 
     assert_raise(EOFError) { @terminal.ask("Any input left?  ", String) }
   end
+
+  def test_indent
+    text = "Testing...\n"
+    @terminal.indent_level=1
+    @terminal.say(text)
+    assert_equal(' '*3+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_level=3
+    @terminal.say(text)
+    assert_equal(' '*9+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_level=0
+    @terminal.indent_size=5
+    @terminal.indent(2, text)
+    assert_equal(' '*10+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_size=4
+    @terminal.indent {
+        @terminal.say(text)
+    }
+    assert_equal(' '*4+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_size=2
+    @terminal.indent(3) { |t|
+        t.say(text)
+    }
+    assert_equal(' '*6+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent { |t|
+        t.indent {
+            t.indent {
+                t.indent { |tt|
+                    tt.say(text)
+                }
+            }
+        }
+    }
+    assert_equal(' '*8+text, @output.string)
+
+    text = "Multi\nLine\nIndentation\n"
+    indent = ' '*4
+    @terminal.indent_level=2
+    @output.truncate(@output.rewind)
+    @terminal.say(text)
+    assert_equal("#{indent}Multi\n#{indent}Line\n#{indent}Indentation\n", @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.multi_indent = false
+    @terminal.say(text)
+    assert_equal("#{indent}Multi\nLine\nIndentation\n", @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent(0, text, true)
+    assert_equal("#{indent}Multi\n#{indent}Line\n#{indent}Indentation\n", @output.string)
+  end
   
+  def test_newline
+    @terminal.newline
+    @terminal.newline
+    assert_equal("\n\n", @output.string)
+  end
+
   def test_bug_fixes
     # auto-complete bug
     @input << "ruby\nRuby\n"
@@ -153,10 +219,10 @@ class TestHighLine < Test::Unit::TestCase
       @input << "\b\b"
       @input.rewind
       answer = @terminal.ask("Please enter your password: ") do |q| 
-          q.echo = "*" 
+        q.echo = "*" 
       end
       assert_equal("", answer)
-      assert_equal("Please enter your password: \n",@output.string)
+      assert_equal("Please enter your password: \n", @output.string)
   end
   
   def test_readline_on_non_echo_question_has_prompt
@@ -167,7 +233,7 @@ class TestHighLine < Test::Unit::TestCase
       q.echo = "*"
     end
     assert_equal("you can't see me", answer)
-    assert_equal("Please enter some hidden text: ****************\n",@output.string)
+    assert_equal("Please enter some hidden text: ****************\n", @output.string)
   end
   
   def test_character_reading
@@ -572,6 +638,14 @@ class TestHighLine < Test::Unit::TestCase
       assert_equal("", result)
     end
   end
+
+  def test_lists_with_nil_items
+    modes = [nil]
+    modes.each do |mode|
+      result = @terminal.list([nil], mode)
+      assert_equal("\n", result)
+    end
+  end
   
   def test_lists_with_one_item
     items = ['Zero']
@@ -831,6 +905,33 @@ class TestHighLine < Test::Unit::TestCase
 
     @terminal.say("This will not have a newline.  ")
     assert_equal("This will not have a newline.  ", @output.string)
+
+    @output.truncate(@output.rewind)
+    
+    @terminal.say("This will not\n end with a newline. ")
+    assert_equal("This will not\n end with a newline. ", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    @terminal.say("This will \nend with a newline.")
+    assert_equal("This will \nend with a newline.\n", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    colorized = @terminal.color("This will not have a newline. ", :green)
+    @terminal.say(colorized)
+    assert_equal("\e[32mThis will not have a newline. \e[0m", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    colorized = @terminal.color("This will have a newline.", :green)
+    @terminal.say(colorized)
+    assert_equal("\e[32mThis will have a newline.\e[0m\n", @output.string)
+  end
+
+  def test_terminal_size
+    assert_instance_of(Fixnum, @terminal.terminal_size[0])
+    assert_instance_of(Fixnum, @terminal.terminal_size[1])
   end
 
   def test_type_conversion
